@@ -221,6 +221,7 @@ ConfigDialog::ConfigDialog(QWidget* parent, QString& caption) :
 				m_totalNumTracksCheckBox = new QCheckBox(i18n("Use &track/total number of tracks format"), v2GroupBox);
 				v2GroupBoxLayout->addWidget(m_totalNumTracksCheckBox, 0, 0, 1, 2);
 #if defined HAVE_ID3LIB || defined HAVE_TAGLIB
+				m_genreNotNumericCheckBox = new QCheckBox(i18n("&Genre as text instead of numeric string"), v2GroupBox);
 				QLabel* textEncodingLabel = new QLabel(i18n("Text &encoding:"), v2GroupBox);
 				m_textEncodingComboBox = new QComboBox(v2GroupBox);
 #endif
@@ -234,6 +235,8 @@ ConfigDialog::ConfigDialog(QWidget* parent, QString& caption) :
 				m_totalNumTracksCheckBox = new QCheckBox(i18n("Use &track/total number of tracks format"), v2GroupBox);
 				v2GroupBox->addSpace(0);
 #if defined HAVE_ID3LIB || defined HAVE_TAGLIB
+				m_genreNotNumericCheckBox = new QCheckBox(i18n("&Genre as text instead of numeric string"), v2GroupBox);
+				v2GroupBox->addSpace(0);
 				QLabel* textEncodingLabel = new QLabel(i18n("Text &encoding:"), v2GroupBox);
 				m_textEncodingComboBox = new QComboBox(v2GroupBox);
 #endif
@@ -250,8 +253,9 @@ ConfigDialog::ConfigDialog(QWidget* parent, QString& caption) :
 					m_textEncodingComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
 					textEncodingLabel->setBuddy(m_textEncodingComboBox);
 #if QT_VERSION >= 0x040000
-					v2GroupBoxLayout->addWidget(textEncodingLabel, 1, 0);
-					v2GroupBoxLayout->addWidget(m_textEncodingComboBox, 1, 1);
+					v2GroupBoxLayout->addWidget(m_genreNotNumericCheckBox, 1, 0, 1, 2);
+					v2GroupBoxLayout->addWidget(textEncodingLabel, 2, 0);
+					v2GroupBoxLayout->addWidget(m_textEncodingComboBox, 2, 1);
 #endif
 				}
 #endif
@@ -262,8 +266,8 @@ ConfigDialog::ConfigDialog(QWidget* parent, QString& caption) :
 					m_id3v2VersionComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
 					id3v2VersionLabel->setBuddy(m_id3v2VersionComboBox);
 #if QT_VERSION >= 0x040000
-					v2GroupBoxLayout->addWidget(id3v2VersionLabel, 2, 0);
-					v2GroupBoxLayout->addWidget(m_id3v2VersionComboBox, 2, 1);
+					v2GroupBoxLayout->addWidget(id3v2VersionLabel, 3, 0);
+					v2GroupBoxLayout->addWidget(m_id3v2VersionComboBox, 3, 1);
 #endif
 				}
 #endif
@@ -344,11 +348,13 @@ ConfigDialog::ConfigDialog(QWidget* parent, QString& caption) :
 #endif
 			if (saveGroupBox) {
 				m_preserveTimeCheckBox = new QCheckBox(i18n("&Preserve file timestamp"), saveGroupBox);
+				m_markChangesCheckBox = new QCheckBox(i18n("&Mark changes"), saveGroupBox);
 #if QT_VERSION >= 0x040000
-				QHBoxLayout* hbox = new QHBoxLayout;
-				hbox->setMargin(2);
-				hbox->addWidget(m_preserveTimeCheckBox);
-				saveGroupBox->setLayout(hbox);
+				QVBoxLayout* vbox = new QVBoxLayout;
+				vbox->setMargin(2);
+				vbox->addWidget(m_preserveTimeCheckBox);
+				vbox->addWidget(m_markChangesCheckBox);
+				saveGroupBox->setLayout(vbox);
 #endif
 				vlayout->addWidget(saveGroupBox);
 			}
@@ -430,11 +436,29 @@ ConfigDialog::ConfigDialog(QWidget* parent, QString& caption) :
 				m_proxyCheckBox = new QCheckBox(i18n("&Proxy:"), proxyGroupBox);
 				m_proxyLineEdit = new QLineEdit(proxyGroupBox);
 #if QT_VERSION >= 0x040000
-				QHBoxLayout* hbox = new QHBoxLayout;
-				hbox->setMargin(2);
-				hbox->addWidget(m_proxyCheckBox);
-				hbox->addWidget(m_proxyLineEdit);
-				proxyGroupBox->setLayout(hbox);
+				m_proxyAuthenticationCheckBox = new QCheckBox(i18n("&Use authentication with proxy"), proxyGroupBox);
+				QLabel* proxyUserNameLabel = new QLabel(i18n("Proxy user &name:"), proxyGroupBox);
+				m_proxyUserNameLineEdit = new QLineEdit(proxyGroupBox);
+				proxyUserNameLabel->setBuddy(m_proxyUserNameLineEdit);
+				QLabel* proxyPasswordLabel = new QLabel(i18n("Proxy pass&word:"), proxyGroupBox);
+				m_proxyPasswordLineEdit = new QLineEdit(proxyGroupBox);
+				proxyPasswordLabel->setBuddy(m_proxyPasswordLineEdit);
+				m_proxyPasswordLineEdit->setEchoMode(QLineEdit::Password);
+				QVBoxLayout* vbox = new QVBoxLayout;
+				vbox->setMargin(2);
+				QHBoxLayout* proxyHbox = new QHBoxLayout;
+				proxyHbox->setMargin(2);
+				proxyHbox->addWidget(m_proxyCheckBox);
+				proxyHbox->addWidget(m_proxyLineEdit);
+				vbox->addLayout(proxyHbox);
+				vbox->addWidget(m_proxyAuthenticationCheckBox);
+				QGridLayout* authLayout = new QGridLayout;
+				authLayout->addWidget(proxyUserNameLabel, 0, 0);
+				authLayout->addWidget(m_proxyUserNameLineEdit, 0, 1);
+				authLayout->addWidget(proxyPasswordLabel, 1, 0);
+				authLayout->addWidget(m_proxyPasswordLineEdit, 1, 1);
+				vbox->addLayout(authLayout);
+				proxyGroupBox->setLayout(vbox);
 #endif
 				vlayout->addWidget(proxyGroupBox);
 			}
@@ -543,6 +567,7 @@ void ConfigDialog::setConfig(const FormatConfig* fnCfg,
 	m_markTruncationsCheckBox->setChecked(miscCfg->m_markTruncations);
 	m_totalNumTracksCheckBox->setChecked(miscCfg->m_enableTotalNumberOfTracks);
 	m_preserveTimeCheckBox->setChecked(miscCfg->m_preserveTime);
+	m_markChangesCheckBox->setChecked(miscCfg->m_markChanges);
 	m_onlyCustomGenresCheckBox->setChecked(miscCfg->m_onlyCustomGenres);
 	m_genresEdit->setStrings(miscCfg->m_customGenres);
 	m_commandsTable->setCommandList(miscCfg->m_contextMenuCommands);
@@ -560,6 +585,7 @@ void ConfigDialog::setConfig(const FormatConfig* fnCfg,
 #endif
 #endif
 #if defined HAVE_ID3LIB || defined HAVE_TAGLIB
+	m_genreNotNumericCheckBox->setChecked(miscCfg->m_genreNotNumeric);
 	int textEncodingV1Index = TextEncodingV1Latin1Index;
 	int index = 0;
 	for (QStringList::const_iterator it = m_textEncodingV1List.begin();
@@ -580,6 +606,11 @@ void ConfigDialog::setConfig(const FormatConfig* fnCfg,
 	m_browserLineEdit->setText(miscCfg->m_browser);
 	m_proxyCheckBox->setChecked(miscCfg->m_useProxy);
 	m_proxyLineEdit->setText(miscCfg->m_proxy);
+#if QT_VERSION >= 0x040000
+	m_proxyAuthenticationCheckBox->setChecked(miscCfg->m_useProxyAuthentication);
+	m_proxyUserNameLineEdit->setText(miscCfg->m_proxyUserName);
+	m_proxyPasswordLineEdit->setText(miscCfg->m_proxyPassword);
+#endif
 #ifndef CONFIG_USE_KDE
 	m_useApplicationFontCheckBox->setChecked(miscCfg->m_useFont);
 	m_applicationFontButton->setEnabled(miscCfg->m_useFont);
@@ -624,6 +655,7 @@ void ConfigDialog::getConfig(FormatConfig* fnCfg,
 	miscCfg->m_markTruncations = m_markTruncationsCheckBox->isChecked();
 	miscCfg->m_enableTotalNumberOfTracks = m_totalNumTracksCheckBox->isChecked();
 	miscCfg->m_preserveTime = m_preserveTimeCheckBox->isChecked();
+	miscCfg->m_markChanges = m_markChangesCheckBox->isChecked();
 	miscCfg->m_onlyCustomGenres = m_onlyCustomGenresCheckBox->isChecked();
 	m_genresEdit->getStrings(miscCfg->m_customGenres);
 	m_commandsTable->getCommandList(miscCfg->m_contextMenuCommands);
@@ -631,6 +663,7 @@ void ConfigDialog::getConfig(FormatConfig* fnCfg,
 	miscCfg->m_commentName = m_commentNameComboBox->currentText();
 #endif
 #if defined HAVE_ID3LIB || defined HAVE_TAGLIB
+	miscCfg->m_genreNotNumeric = m_genreNotNumericCheckBox->isChecked();
 	miscCfg->m_textEncodingV1 =
 		getTextEncodingV1CodecName(m_textEncodingV1ComboBox->currentText());
 	miscCfg->m_textEncoding = m_textEncodingComboBox->QCM_currentIndex();
@@ -641,6 +674,11 @@ void ConfigDialog::getConfig(FormatConfig* fnCfg,
 	miscCfg->m_browser = m_browserLineEdit->text();
 	miscCfg->m_useProxy = m_proxyCheckBox->isChecked();
 	miscCfg->m_proxy = m_proxyLineEdit->text();
+#if QT_VERSION >= 0x040000
+	miscCfg->m_useProxyAuthentication = m_proxyAuthenticationCheckBox->isChecked();
+	miscCfg->m_proxyUserName = m_proxyUserNameLineEdit->text();
+	miscCfg->m_proxyPassword = m_proxyPasswordLineEdit->text();
+#endif
 #ifndef CONFIG_USE_KDE
 	if (m_useApplicationFontCheckBox->isChecked()) {
 		QFont font = QApplication::font();
