@@ -35,6 +35,9 @@
 #if QT_VERSION >= 0x040000
 #include <QMenu>
 #include <QHeaderView>
+#if QT_VERSION >= 0x040300
+#include <QDirIterator>
+#endif
 #else
 #include <qpopupmenu.h>
 #include <qheader.h>
@@ -63,6 +66,11 @@
 #include "taglibfile.h"
 #endif
 
+/** Only defined for generation of KDE3 translation files */
+#define FOR_KDE3_PO_1 I18N_NOOP("Do you really want to delete these %1 items?")
+/** Only defined for generation of KDE3 translation files */
+#define FOR_KDE3_PO_2 I18N_NOOP("Error while deleting these %1 items:")
+
 /**
  * Replaces context command format codes in a string.
  */
@@ -715,10 +723,14 @@ void FileList::readSubDirectory(DirInfo* dirInfo, FileListItem* item,
 	int numFiles = 0;
 	FileListItem* last = 0;
 	QDir dir(dirname);
-	QStringList dirContents = dir.entryList(QDir::Dirs) +
 #if QT_VERSION >= 0x040000
-		dir.entryList(Kid3App::s_miscCfg.m_nameFilter.split(' '), QDir::Files);
+	const QDir::Filters dirFilters =
+		QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Files;
+	QStringList nameFilters(Kid3App::s_miscCfg.m_nameFilter.split(' '));
+	QStringList dirContents = dir.entryList(
+		nameFilters, dirFilters, QDir::DirsFirst | QDir::IgnoreCase);
 #else
+	QStringList dirContents = dir.entryList(QDir::Dirs) +
 		dir.entryList(Kid3App::s_miscCfg.m_nameFilter, QDir::Files);
 #endif
 	for (QStringList::Iterator it = dirContents.begin();
@@ -753,6 +765,12 @@ void FileList::readSubDirectory(DirInfo* dirInfo, FileListItem* item,
 				}
 				if (last) {
 					last->setDirInfo(new DirInfo(filename));
+#if QT_VERSION >= 0x040300
+					last->setChildIndicatorPolicy(
+						QDirIterator(filename, nameFilters, dirFilters).hasNext() ?
+						QTreeWidgetItem::ShowIndicator :
+						QTreeWidgetItem::DontShowIndicatorWhenChildless);
+#endif
 				}
 			}
 		}
@@ -1010,7 +1028,7 @@ void FileList::executeContextCommand(int id)
 void FileList::executeAction(QAction* action)
 {
 	if (action) {
-		QString name = action->text();
+		QString name = action->text().remove('&');
 		int id = 0;
 		for (MiscConfig::MenuCommandList::const_iterator
 					 it = Kid3App::s_miscCfg.m_contextMenuCommands.begin();
