@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 9 Jan 2003
  *
- * Copyright (C) 2003-2008  Urs Fleisch
+ * Copyright (C) 2003-2009  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -86,10 +86,12 @@
 #include "frametable.h"
 #include "configdialog.h"
 #include "importdialog.h"
+#include "browsecoverartdialog.h"
 #include "exportdialog.h"
 #include "numbertracksdialog.h"
 #include "filterdialog.h"
 #include "rendirdialog.h"
+#include "downloaddialog.h"
 #include "filelistitem.h"
 #include "dirlist.h"
 #include "pictureframe.h"
@@ -240,8 +242,10 @@ QString Kid3App::s_dirName;
  * Constructor.
  */
 Kid3App::Kid3App() :
-	m_importDialog(0), m_exportDialog(0), m_renDirDialog(0),
-	m_numberTracksDialog(0), m_filterDialog(0)
+	m_downloadToAllFilesInDir(false),
+	m_importDialog(0), m_browseCoverArtDialog(0),
+	m_exportDialog(0), m_renDirDialog(0),
+	m_numberTracksDialog(0), m_filterDialog(0), m_downloadDialog(0)
 {
 #ifdef CONFIG_USE_KDE
 #if KDE_VERSION >= 0x035c00
@@ -260,7 +264,12 @@ Kid3App::Kid3App() :
 #if !defined _WIN32 && !defined WIN32 && defined CFG_DATAROOTDIR
 	QPixmap icon;
 	if (icon.load(QString(CFG_DATAROOTDIR) +
-								"/icons/hicolor/48x48/apps/kid3-qt.png")) {
+#ifndef __APPLE__
+								"/icons/hicolor/48x48/apps/kid3-qt.png"
+#else
+								"/kid3.png"
+#endif
+				)) {
 		QCM_setWindowIcon(icon);
 	}
 #endif
@@ -306,6 +315,8 @@ Kid3App::~Kid3App()
 	delete m_renDirDialog;
 	delete m_numberTracksDialog;
 	delete m_filterDialog;
+	delete m_downloadDialog;
+	delete m_browseCoverArtDialog;
 #ifndef CONFIG_USE_KDE
 	delete s_helpBrowser;
 	s_helpBrowser = 0;
@@ -363,7 +374,11 @@ void Kid3App::initActions()
 	fileSave->KCM_setStatusTip(i18n("Saves the changed files"));
 	fileQuit->KCM_setStatusTip(i18n("Quits the application"));
 	editSelectAll->KCM_setStatusTip(i18n("Select all files"));
+#if KDE_VERSION >= 0x035c00
+	editSelectAll->setShortcut(KShortcut("Alt+Shift+A"));
+#else
 	editSelectAll->setShortcut(KShortcut("Alt+A"));
+#endif
 	editDeselect->KCM_setStatusTip(i18n("Deselect all files"));
 	settingsShortcuts->KCM_setStatusTip(i18n("Configure Shortcuts"));
 	settingsToolbars->KCM_setStatusTip(i18n("Configure Toolbars"));
@@ -399,6 +414,10 @@ void Kid3App::initActions()
 		    SLOT(slotImportMusicBrainz()), actionCollection(),
 		    "import_musicbrainz");
 #endif
+	KCM_KAction(fileBrowseCoverArt,
+		    i18n("&Browse Cover Art..."), this,
+		    SLOT(slotBrowseCoverArt()), actionCollection(),
+		    "browse_cover_art");
 	KCM_KActionIcon(fileExport, KCM_ICON_document_export,
 		    i18n("&Export..."), this,
 		    SLOT(slotExport()), actionCollection(),
@@ -447,6 +466,10 @@ void Kid3App::initActions()
 		            i18n("Hide Tag &2"), this,
 								SLOT(slotSettingsShowHideV2()), actionCollection(),
 								"hide_v2");
+	KCM_KActionVar(m_settingsShowHidePicture,
+		            i18n("Hide &Picture"), this,
+								SLOT(slotSettingsShowHidePicture()), actionCollection(),
+								"hide_picture");
 
 	KCM_KActionShortcutIcon(editPreviousFile, KShortcut("Alt+Up"), KCM_ICON_go_previous,
 		    i18n("&Previous File"), m_view,
@@ -590,6 +613,13 @@ void Kid3App::initActions()
 			this, SLOT(slotImportMusicBrainz()));
 	}
 #endif
+	QAction* fileBrowseCoverArt = new QAction(this);
+	if (fileBrowseCoverArt) {
+		fileBrowseCoverArt->setStatusTip(i18n("Browse album cover artwork"));
+		fileBrowseCoverArt->QCM_setMenuText(i18n("&Browse Cover Art..."));
+		connect(fileBrowseCoverArt, QCM_SIGNAL_triggered,
+			this, SLOT(slotBrowseCoverArt()));
+	}
 	QAction* fileExport = new QAction(this);
 	if (fileExport) {
 		fileExport->setStatusTip(i18n("Export to file or clipboard"));
@@ -732,6 +762,13 @@ void Kid3App::initActions()
 		connect(m_settingsShowHideV2, QCM_SIGNAL_triggered,
 			this, SLOT(slotSettingsShowHideV2()));
 	}
+	m_settingsShowHidePicture = new QAction(this);
+	if (m_settingsShowHidePicture) {
+		m_settingsShowHidePicture->setStatusTip(i18n("Hide Picture"));
+		m_settingsShowHidePicture->QCM_setMenuText(i18n("Hide &Picture"));
+		connect(m_settingsShowHidePicture, QCM_SIGNAL_triggered,
+			this, SLOT(slotSettingsShowHidePicture()));
+	}
 	QAction* settingsConfigure = new QAction(this);
 	if (settingsConfigure) {
 		settingsConfigure->setStatusTip(i18n("Configure Kid3"));
@@ -774,6 +811,7 @@ void Kid3App::initActions()
 #ifdef HAVE_TUNEPIMP
 		QCM_addAction(fileMenu, fileImportMusicBrainz);
 #endif
+		QCM_addAction(fileMenu, fileBrowseCoverArt);
 		QCM_addAction(fileMenu, fileExport);
 		QCM_addAction(fileMenu, fileCreatePlaylist);
 		fileMenu->QCM_addSeparator();
@@ -798,6 +836,7 @@ void Kid3App::initActions()
 
 		QCM_addAction(settingsMenu, m_settingsShowHideV1);
 		QCM_addAction(settingsMenu, m_settingsShowHideV2);
+		QCM_addAction(settingsMenu, m_settingsShowHidePicture);
 		settingsMenu->QCM_addSeparator();
 		QCM_addAction(settingsMenu, settingsConfigure);
 
@@ -925,6 +964,8 @@ void Kid3App::saveOptions()
 	m_fileOpenRecent->saveEntries(m_config, "Recent Files");
 #endif
 #else
+	s_miscCfg.m_windowX = x();
+	s_miscCfg.m_windowY = y();
 	s_miscCfg.m_windowWidth = size().width();
 	s_miscCfg.m_windowHeight = size().height();
 #endif
@@ -993,6 +1034,7 @@ void Kid3App::readOptions()
 #endif
 	updateHideV1();
 	updateHideV2();
+	updateHidePicture();
 #ifdef CONFIG_USE_KDE
 	setAutoSaveSettings();
 #if KDE_VERSION >= 0x035c00
@@ -1008,6 +1050,9 @@ void Kid3App::readOptions()
 	if (s_miscCfg.m_windowWidth != -1 && s_miscCfg.m_windowHeight != -1) {
 		resize(s_miscCfg.m_windowWidth, s_miscCfg.m_windowHeight);
 	}
+	if (s_miscCfg.m_windowX != -1 && s_miscCfg.m_windowY != -1) {
+		move(s_miscCfg.m_windowX, s_miscCfg.m_windowY);
+	}
 #endif
 	m_view->readConfig();
 }
@@ -1669,7 +1714,7 @@ void Kid3App::slotHelpAbout()
 	QMessageBox::about(
 		(Kid3App*)parent(), "Kid3",
 		"Kid3 " VERSION
-		"\n(c) 2003-2008 Urs Fleisch\nufleisch@users.sourceforge.net");
+		"\n(c) 2003-2009 Urs Fleisch\nufleisch@users.sourceforge.net");
 }
 
 /**
@@ -1815,6 +1860,11 @@ void Kid3App::getTagsFromImportDialog(bool destV1, bool destV2)
 	}
 	slotStatusMsg(i18n("Ready."));
 	QApplication::restoreOverrideCursor();
+
+	if (destV2 && flt.isEnabled(Frame::FT_Picture) &&
+	    !m_trackDataList.getCoverArtUrl().isEmpty()) {
+		downloadImage(m_trackDataList.getCoverArtUrl(), true);
+	}
 }
 
 /**
@@ -1929,6 +1979,33 @@ void Kid3App::slotImportMusicBrainz()
 #endif
 }
 
+/**
+ * Browse album cover artwork.
+ */
+void Kid3App::slotBrowseCoverArt()
+{
+	if (!m_browseCoverArtDialog) {
+		m_browseCoverArtDialog = new BrowseCoverArtDialog(0);
+	}
+	if (m_browseCoverArtDialog) {
+		FrameCollection frames2;
+		FileListItem* item;
+		TaggedFile* taggedFile;
+		if ((item = m_view->currentFile()) != 0 &&
+				(taggedFile = item->getFile()) != 0) {
+			taggedFile->readTags(false);
+			FrameCollection frames1;
+			taggedFile->getAllFramesV1(frames1);
+			taggedFile->getAllFramesV2(frames2);
+			frames2.merge(frames1);
+		}
+
+		m_browseCoverArtDialog->readConfig();
+		m_browseCoverArtDialog->setFrames(frames2);
+		m_browseCoverArtDialog->exec();
+	}
+}
+
 #if defined HAVE_ID3LIB && defined HAVE_TAGLIB
 /**
  * Read file with TagLib if it has an ID3v2.4 tag.
@@ -1981,6 +2058,9 @@ void Kid3App::setExportData(int src)
 				trackData.setFileExtension(taggedFile->getFileExtension());
 				trackData.setTagFormatV1(taggedFile->getTagFormatV1());
 				trackData.setTagFormatV2(taggedFile->getTagFormatV2());
+				TaggedFile::DetailInfo info;
+				taggedFile->getDetailInfo(info);
+				trackData.setDetailInfo(info);
 				if (src == ExportDialog::SrcV1) {
 					taggedFile->getAllFramesV1(trackData);
 				} else {
@@ -2095,6 +2175,31 @@ void Kid3App::updateHideV2()
 #endif
 }
 
+/**
+ * Show or hide the picture according to the settings and
+ * set the menu entries appropriately.
+ */
+void Kid3App::updateHidePicture()
+{
+	m_view->hidePicture(s_miscCfg.m_hidePicture);
+	if (s_miscCfg.m_hidePicture) {
+#ifdef CONFIG_USE_KDE
+		m_settingsShowHidePicture->setText(i18n("Show &Picture"));
+#else
+		m_settingsShowHidePicture->setStatusTip(i18n("Show Picture"));
+		m_settingsShowHidePicture->QCM_setMenuText(i18n("Show &Picture"));
+#endif
+
+	} else {
+#ifdef CONFIG_USE_KDE
+		m_settingsShowHidePicture->setText(i18n("Hide &Picture"));
+#else
+		m_settingsShowHidePicture->setStatusTip(i18n("Hide Picture"));
+		m_settingsShowHidePicture->QCM_setMenuText(i18n("Hide &Picture"));
+#endif
+	}
+}
+
 /**
  * Show or hide ID3v1.1 controls.
  */
@@ -2113,6 +2218,24 @@ void Kid3App::slotSettingsShowHideV2()
 	updateHideV2();
 }
 
+/**
+ * Show or hide picture.
+ */
+void Kid3App::slotSettingsShowHidePicture()
+{
+	s_miscCfg.m_hidePicture = !s_miscCfg.m_hidePicture;
+	updateHidePicture();
+#if QT_VERSION >= 0x040000
+	// In Qt3 the picture is displayed too small if Kid3 is started with picture
+	// hidden, and then "Show Picture" is triggered while a file with a picture
+	// is selected. Thus updating the controls is only done for Qt4, in Qt3 the
+	// file has to be selected again for the picture to be shown.
+	if (!s_miscCfg.m_hidePicture) {
+		updateGuiControls();
+	}
+#endif
+}
+
 /**
  * Preferences.
  */
@@ -2140,6 +2263,11 @@ void Kid3App::slotSettingsConfigure()
 			if (!s_miscCfg.m_markTruncations) {
 				m_view->frameTableV1()->markRows(0);
 			}
+			if (!s_miscCfg.m_markChanges) {
+				m_view->frameTableV1()->markChangedFrames(0);
+				m_view->frameTableV2()->markChangedFrames(0);
+				m_view->markChangedFilename(false);
+			}
 			setTextEncodings();
 #if QT_VERSION < 0x040000
 			m_view->frameTableV1()->triggerUpdateGenres();
@@ -2157,9 +2285,7 @@ void Kid3App::slotSettingsConfigure()
  */
 void Kid3App::slotApplyFilenameFormat()
 {
-	if (m_view->numFilesSelected() == 1) {
-		updateCurrentSelection();
-	}
+	updateCurrentSelection(true);
 	FileListItem* mp3file = m_view->firstFile();
 	bool no_selection = m_view->numFilesSelected() == 0;
 	while (mp3file != 0) {
@@ -2181,9 +2307,7 @@ void Kid3App::slotApplyFilenameFormat()
 void Kid3App::slotApplyId3Format()
 {
 	FrameCollection frames;
-	if (m_view->numFilesSelected() == 1) {
-		updateCurrentSelection();
-	}
+	updateCurrentSelection(true);
 	FrameFilter fltV1(m_view->frameTableV1()->getEnabledFrameFilter(true));
 	FrameFilter fltV2(m_view->frameTableV2()->getEnabledFrameFilter(true));
 	FileListItem* mp3file = m_view->firstFile();
@@ -2343,7 +2467,7 @@ void Kid3App::slotRenameDirectory()
  */
 void Kid3App::numberTracks(int nr, bool destV1, bool destV2)
 {
-	updateCurrentSelection();
+	updateCurrentSelection(true);
 	FileListItem* mp3file = m_view->firstFileInDir();
 	bool no_selection = m_view->numFilesSelected() == 0;
 	while (mp3file != 0) {
@@ -2498,9 +2622,7 @@ void Kid3App::slotFilter()
 void Kid3App::slotConvertToId3v24()
 {
 #ifdef HAVE_TAGLIB
-	if (m_view->numFilesSelected() == 1) {
-		updateCurrentSelection();
-	}
+	updateCurrentSelection(true);
 	FileListItem* item = m_view->firstFile();
 	while (item != 0) {
 		TaggedFile* taggedFile;
@@ -2554,9 +2676,7 @@ void Kid3App::slotConvertToId3v24()
 void Kid3App::slotConvertToId3v23()
 {
 #if defined HAVE_TAGLIB && defined HAVE_ID3LIB
-	if (m_view->numFilesSelected() == 1) {
-		updateCurrentSelection();
-	}
+	updateCurrentSelection(true);
 	FileListItem* item = m_view->firstFile();
 	while (item != 0) {
 		TaggedFile* taggedFile;
@@ -2621,6 +2741,7 @@ void Kid3App::openDrop(QString txt)
 		QString dir = url.path().QCM_trimmed();
 #endif
 		if (dir.endsWith(".jpg", QCM_CaseInsensitive) ||
+				dir.endsWith(".jpeg", QCM_CaseInsensitive) ||
 				dir.endsWith(".png", QCM_CaseInsensitive)) {
 			PictureFrame frame;
 			if (PictureFrame::setDataFromFile(frame, dir)) {
@@ -2632,6 +2753,7 @@ void Kid3App::openDrop(QString txt)
 				PictureFrame::setMimeTypeFromFileName(frame, fileName);
 				PictureFrame::setDescription(frame, fileName);
 				addFrame(&frame);
+				updateGuiControls();
 			}
 		} else {
 			updateCurrentSelection();
@@ -2651,10 +2773,80 @@ void Kid3App::dropImage(const QImage& image)
 		PictureFrame frame;
 		if (PictureFrame::setDataFromImage(frame, image)) {
 			addFrame(&frame);
+			updateGuiControls();
 		}
 	}
 }
 
+/**
+ * Download an image file.
+ *
+ * @param url           URL of image
+ * @param allFilesInDir true to add the image to all files in the directory
+ */
+void Kid3App::downloadImage(const QString& url, bool allFilesInDir)
+{
+	QString imgurl(BrowseCoverArtDialog::getImageUrl(url));
+	if (!imgurl.isEmpty()) {
+		if (!m_downloadDialog) {
+			m_downloadDialog = new DownloadDialog(0, i18n("Download"));
+			connect(m_downloadDialog, SIGNAL(downloadFinished(const QByteArray&, const QString&, const QString&)),
+							this, SLOT(imageDownloaded(const QByteArray&, const QString&, const QString&)));
+		}
+		if (m_downloadDialog) {
+			int hostPos = imgurl.QCM_indexOf("://");
+			if (hostPos > 0) {
+				int pathPos = imgurl.QCM_indexOf("/", hostPos + 3);
+				if (pathPos > hostPos) {
+					m_downloadToAllFilesInDir = allFilesInDir;
+					m_downloadDialog->startDownload(
+						imgurl.mid(hostPos + 3, pathPos - hostPos - 3),
+						imgurl.mid(pathPos));
+					m_downloadDialog->exec();
+				}
+			}
+		}
+	}
+}
+
+/**
+ * Handle URL on drop.
+ *
+ * @param txt dropped URL.
+ */
+void Kid3App::dropUrl(const QString& txt)
+{
+	downloadImage(txt, false);
+}
+
+/**
+ * Add a downloaded image.
+ *
+ * @param data     HTTP response of download
+ * @param mimeType MIME type of data
+ * @param url      URL of downloaded data
+ */
+void Kid3App::imageDownloaded(const QByteArray& data,
+                              const QString& mimeType, const QString& url)
+{
+	if (mimeType.startsWith("image")) {
+		PictureFrame frame(data, url, PictureFrame::PT_CoverFront, mimeType);
+		if (m_downloadToAllFilesInDir) {
+			FileListItem* mp3file = m_view->firstFileInDir();
+			while (mp3file != 0) {
+				TaggedFile* taggedFile = mp3file->getFile();
+				taggedFile->readTags(false);
+				taggedFile->addFrameV2(frame);
+				mp3file = m_view->nextFileInDir();
+			}
+			m_downloadToAllFilesInDir = false;
+		} else {
+			addFrame(&frame);
+		}
+		updateGuiControls();
+	}
+}
+
 /**
  * Update modification state, caption and listbox entries.
  */
@@ -2681,8 +2873,11 @@ void Kid3App::updateModificationState()
 
 /**
  * Update files of current selection.
+ *
+ * @param onlyIfSingleFileSelected if true, the selection is only updated
+ *                                 if a single file is selected
  */
-void Kid3App::updateCurrentSelection()
+void Kid3App::updateCurrentSelection(bool onlyIfSingleFileSelected)
 {
 	int numFiles = 0;
 	FileListItem* mp3file = m_view->firstFile();
@@ -2695,6 +2890,9 @@ void Kid3App::updateCurrentSelection()
 		}
 		mp3file = m_view->nextFile();
 	}
+	if (onlyIfSingleFileSelected && numFiles != 1) {
+		return;
+	}
 	if (numFiles > 0) {
 		m_view->frameTableV1()->tableToFrames(numFiles > 1);
 		m_view->frameTableV2()->tableToFrames(numFiles > 1);
@@ -2792,6 +2990,24 @@ void Kid3App::updateGuiControls()
 		if (s_miscCfg.m_markTruncations) {
 			m_view->frameTableV1()->markRows(single_v2_file->getTruncationFlags());
 		}
+		if (s_miscCfg.m_markChanges) {
+			m_view->frameTableV1()->markChangedFrames(
+				single_v2_file->getChangedFramesV1());
+			m_view->frameTableV2()->markChangedFrames(
+				single_v2_file->getChangedFramesV2());
+			m_view->markChangedFilename(single_v2_file->isFilenameChanged());
+		}
+
+		if (!s_miscCfg.m_hidePicture) {
+			FrameCollection::const_iterator it =
+				m_view->frameTableV2()->frames().find(Frame(Frame::FT_Picture, "", "", -1));
+			if (it == m_view->frameTableV2()->frames().end()) {
+				m_view->setPictureData(0);
+			} else {
+				QByteArray data;
+				m_view->setPictureData(PictureFrame::getData(*it, data) ? &data : 0);
+			}
+		}
 	}
 	else {
 		m_view->setFilenameEditEnabled(false);
@@ -2802,6 +3018,14 @@ void Kid3App::updateGuiControls()
 		if (s_miscCfg.m_markTruncations) {
 			m_view->frameTableV1()->markRows(0);
 		}
+		if (s_miscCfg.m_markChanges) {
+			m_view->frameTableV1()->markChangedFrames(0);
+			m_view->frameTableV2()->markChangedFrames(0);
+			m_view->markChangedFilename(false);
+		}
+		if (!s_miscCfg.m_hidePicture) {
+			m_view->setPictureData(0);
+		}
 	}
 	m_view->frameTableV1()->setAllCheckBoxes(num_files_selected == 1);
 	m_view->frameTableV1()->framesToTable();
@@ -2832,7 +3056,7 @@ void Kid3App::fileSelected()
  */
 void Kid3App::copyTagsV1()
 {
-	updateCurrentSelection();
+	updateCurrentSelection(true);
 	m_copyTags = m_view->frameTableV1()->frames().copyEnabledFrames(
 		m_view->frameTableV1()->getEnabledFrameFilter(true));
 }
@@ -2842,7 +3066,7 @@ void Kid3App::copyTagsV1()
  */
 void Kid3App::copyTagsV2()
 {
-	updateCurrentSelection();
+	updateCurrentSelection(true);
 	m_copyTags = m_view->frameTableV2()->frames().copyEnabledFrames(
 		m_view->frameTableV2()->getEnabledFrameFilter(true));
 }
@@ -2852,7 +3076,7 @@ void Kid3App::copyTagsV2()
  */
 void Kid3App::pasteTagsV1()
 {
-	updateCurrentSelection();
+	updateCurrentSelection(true);
 	FrameCollection frames(m_copyTags.copyEnabledFrames(
 													 m_view->frameTableV1()->getEnabledFrameFilter(true)));
 	formatFramesIfEnabled(frames);
@@ -2872,7 +3096,7 @@ void Kid3App::pasteTagsV1()
  */
 void Kid3App::pasteTagsV2()
 {
-	updateCurrentSelection();
+	updateCurrentSelection(true);
 	FrameCollection frames(m_copyTags.copyEnabledFrames(
 													 m_view->frameTableV2()->getEnabledFrameFilter(true)));
 	formatFramesIfEnabled(frames);
@@ -2894,7 +3118,7 @@ void Kid3App::pasteTagsV2()
  */
 void Kid3App::getTagsFromFilenameV1()
 {
-	updateCurrentSelection();
+	updateCurrentSelection(true);
 	FrameCollection frames;
 	FileListItem* mp3file = m_view->firstFile();
 	bool multiselect = m_view->numFilesSelected() > 1;
@@ -2924,7 +3148,7 @@ void Kid3App::getTagsFromFilenameV1()
  */
 void Kid3App::getTagsFromFilenameV2()
 {
-	updateCurrentSelection();
+	updateCurrentSelection(true);
 	FrameCollection frames;
 	FileListItem* mp3file = m_view->firstFile();
 	bool multiselect = m_view->numFilesSelected() > 1;
@@ -2989,7 +3213,7 @@ void Kid3App::getFilenameFromTags(int tag_version)
  */
 void Kid3App::copyV1ToV2()
 {
-	updateCurrentSelection();
+	updateCurrentSelection(true);
 	FrameCollection frames;
 	FrameFilter flt(m_view->frameTableV2()->getEnabledFrameFilter(true));
 	FileListItem* mp3file = m_view->firstFile();
@@ -3011,7 +3235,7 @@ void Kid3App::copyV1ToV2()
  */
 void Kid3App::copyV2ToV1()
 {
-	updateCurrentSelection();
+	updateCurrentSelection(true);
 	FrameCollection frames;
 	FrameFilter flt(m_view->frameTableV1()->getEnabledFrameFilter(true));
 	FileListItem* mp3file = m_view->firstFile();
@@ -3204,6 +3428,10 @@ void Kid3App::addFrame(const Frame* frame)
 			if (m_framelist->selectFrame() &&
 					m_framelist->addFrame(true)) {
 				updateAfterFrameModification(taggedFile);
+				if (m_framelist->isPictureFrame()) {
+					// update preview picture
+					updateGuiControls();
+				}
 			}
 		} else {
 			m_framelist->setFrame(*frame);
