@@ -55,7 +55,7 @@ int main(int argc, char* argv[])
 	KAboutData aboutData(
 		"kid3", 0, ki18n("Kid3"),
 		VERSION, ki18n(description), KAboutData::License_GPL,
-		ki18n("(c) 2003-2008 Urs Fleisch"), KLocalizedString(), "http://kid3.sourceforge.net",
+		ki18n("(c) 2003-2009 Urs Fleisch"), KLocalizedString(), "http://kid3.sourceforge.net",
 		"ufleisch@users.sourceforge.net");
 	aboutData.addAuthor(ki18n("Urs Fleisch"), KLocalizedString(), "ufleisch@users.sourceforge.net");
 	KCmdLineArgs::init(argc, argv, &aboutData);
@@ -108,7 +108,7 @@ int main(int argc, char* argv[])
 	KAboutData aboutData(
 	    "kid3", I18N_NOOP("Kid3"),
 	    VERSION, description, KAboutData::License_GPL,
-	    "(c) 2003-2008 Urs Fleisch", 0, "http://kid3.sourceforge.net",
+	    "(c) 2003-2009 Urs Fleisch", 0, "http://kid3.sourceforge.net",
 	    "ufleisch@users.sourceforge.net");
 	aboutData.addAuthor("Urs Fleisch",0, "ufleisch@users.sourceforge.net");
 	KCmdLineArgs::init(argc, argv, &aboutData);
@@ -142,6 +142,7 @@ int main(int argc, char* argv[])
 #include <qapplication.h>
 #include <qlocale.h>
 #include <qtranslator.h>
+#include <qdir.h>
 
 #include "kid3.h"
 
@@ -175,6 +176,13 @@ int main(int argc, char* argv[])
 	kid3_tr.load( QString("kid3_") + locale, "." );
 	app.installTranslator(&kid3_tr);
 
+#ifdef __APPLE__
+ QDir dir(QApplication::applicationDirPath());
+ dir.cdUp();
+ dir.cd("PlugIns");
+ QApplication::setLibraryPaths(QStringList(dir.absolutePath()));
+#endif
+
 	Kid3App* kid3 = new Kid3App();
 	if (kid3) {
 		kid3->show();
