@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 17 Jan 2004
  *
- * Copyright (C) 2004-2007  Urs Fleisch
+ * Copyright (C) 2004-2009  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -45,7 +45,11 @@
 const char* const MiscConfig::s_defaultCommentName = "COMMENT";
 
 /** Default value for web browser */
+#ifdef __APPLE__
+const char* const MiscConfig::s_defaultBrowser = "open";
+#else
 const char* const MiscConfig::s_defaultBrowser = "xdg-open";
+#endif
 
 /** Default filename format list */
 static const char* fnFmt[] = {
@@ -99,7 +103,9 @@ MiscConfig::MiscConfig(const QString& group) :
 	GeneralConfig(group),
 	m_markTruncations(true),
 	m_enableTotalNumberOfTracks(false),
+	m_genreNotNumeric(false),
 	m_preserveTime(false),
+	m_markChanges(true),
 	m_commentName(s_defaultCommentName),
 	m_nameFilter(""),
 	m_formatText(s_defaultFnFmtList[0]),
@@ -109,13 +115,18 @@ MiscConfig::MiscConfig(const QString& group) :
 	m_renDirSrc(0),
 	m_hideV1(false),
 	m_hideV2(false),
+	m_hidePicture(false),
 	m_id3v2Version(ID3v2_3_0),
 	m_textEncodingV1(""),
 	m_textEncoding(TE_ISO8859_1),
 	m_useProxy(false),
+#if QT_VERSION >= 0x040000
+	m_useProxyAuthentication(false),
+#endif
 	m_onlyCustomGenres(false)
 #ifndef CONFIG_USE_KDE
-	, m_windowWidth(-1), m_windowHeight(-1), m_useFont(false), m_fontSize(-1)
+	, m_windowX(-1), m_windowY(-1), m_windowWidth(-1), m_windowHeight(-1),
+	m_useFont(false), m_fontSize(-1)
 #endif
 {
 }
@@ -148,18 +159,26 @@ void MiscConfig::writeToConfig(
 	cfg.writeEntry("RenameDirectorySource", m_renDirSrc);
 	cfg.writeEntry("MarkTruncations", m_markTruncations);
 	cfg.writeEntry("EnableTotalNumberOfTracks", m_enableTotalNumberOfTracks);
+	cfg.writeEntry("GenreNotNumeric", m_genreNotNumeric);
 	cfg.writeEntry("PreserveTime", m_preserveTime);
+	cfg.writeEntry("MarkChanges", m_markChanges);
 	cfg.writeEntry("CommentName", m_commentName);
 	cfg.writeEntry("SplitterSizes", m_splitterSizes);
 	cfg.writeEntry("VSplitterSizes", m_vSplitterSizes);
 	cfg.writeEntry("CustomGenres", m_customGenres);
 	cfg.writeEntry("HideV1", m_hideV1);
 	cfg.writeEntry("HideV2", m_hideV2);
+	cfg.writeEntry("HidePicture", m_hidePicture);
 	cfg.writeEntry("ID3v2Version", m_id3v2Version);
 	cfg.writeEntry("TextEncodingV1", m_textEncodingV1);
 	cfg.writeEntry("TextEncoding", m_textEncoding);
 	cfg.writeEntry("UseProxy", m_useProxy);
 	cfg.writeEntry("Proxy", m_proxy);
+#if QT_VERSION >= 0x040000
+	cfg.writeEntry("UseProxyAuthentication", m_useProxyAuthentication);
+	cfg.writeEntry("ProxyUserName", m_proxyUserName);
+	cfg.writeEntry("ProxyPassword", m_proxyPassword);
+#endif
 	cfg.writeEntry("Browser", m_browser);
 	cfg.writeEntry("OnlyCustomGenres", m_onlyCustomGenres);
 
@@ -190,7 +209,9 @@ void MiscConfig::writeToConfig(
 	config->QCM_writeEntry("/RenameDirectorySource", m_renDirSrc);
 	config->QCM_writeEntry("/MarkTruncations", m_markTruncations);
 	config->QCM_writeEntry("/EnableTotalNumberOfTracks", m_enableTotalNumberOfTracks);
+	config->QCM_writeEntry("/GenreNotNumeric", m_genreNotNumeric);
 	config->QCM_writeEntry("/PreserveTime", m_preserveTime);
+	config->QCM_writeEntry("/MarkChanges", m_markChanges);
 	config->QCM_writeEntry("/CommentName", m_commentName);
 
 #if QT_VERSION >= 0x040000
@@ -212,13 +233,21 @@ void MiscConfig::writeToConfig(
 	config->QCM_writeEntry("/CustomGenres", m_customGenres);
 	config->QCM_writeEntry("/HideV1", m_hideV1);
 	config->QCM_writeEntry("/HideV2", m_hideV2);
+	config->QCM_writeEntry("/HidePicture", m_hidePicture);
 	config->QCM_writeEntry("/ID3v2Version", m_id3v2Version);
 	config->QCM_writeEntry("/TextEncodingV1", m_textEncodingV1);
 	config->QCM_writeEntry("/TextEncoding", m_textEncoding);
 	config->QCM_writeEntry("/UseProxy", m_useProxy);
 	config->QCM_writeEntry("/Proxy", m_proxy);
+#if QT_VERSION >= 0x040000
+	config->QCM_writeEntry("/UseProxyAuthentication", m_useProxyAuthentication);
+	config->QCM_writeEntry("/ProxyUserName", m_proxyUserName);
+	config->QCM_writeEntry("/ProxyPassword", m_proxyPassword);
+#endif
 	config->QCM_writeEntry("/Browser", m_browser);
 	config->QCM_writeEntry("/OnlyCustomGenres", m_onlyCustomGenres);
+	config->QCM_writeEntry("/WindowX", m_windowX);
+	config->QCM_writeEntry("/WindowY", m_windowY);
 	config->QCM_writeEntry("/WindowWidth", m_windowWidth);
 	config->QCM_writeEntry("/WindowHeight", m_windowHeight);
 	config->QCM_writeEntry("/UseFont", m_useFont);
@@ -272,7 +301,9 @@ void MiscConfig::readFromConfig(
 	m_renDirSrc = cfg.KCM_readBoolEntry("RenameDirectorySource", m_renDirSrc);
 	m_markTruncations = cfg.KCM_readBoolEntry("MarkTruncations", m_markTruncations);
 	m_enableTotalNumberOfTracks = cfg.KCM_readBoolEntry("EnableTotalNumberOfTracks", m_enableTotalNumberOfTracks);
+	m_genreNotNumeric = cfg.KCM_readBoolEntry("GenreNotNumeric", m_genreNotNumeric);
 	m_preserveTime = cfg.KCM_readBoolEntry("PreserveTime", m_preserveTime);
+	m_markChanges = cfg.KCM_readBoolEntry("MarkChanges", m_markChanges);
 	m_commentName = cfg.readEntry("CommentName", s_defaultCommentName);
 	m_formatText =
 	    cfg.readEntry("FormatText2", s_defaultFnFmtList[0]);
@@ -283,11 +314,17 @@ void MiscConfig::readFromConfig(
 	m_customGenres = cfg.KCM_readListEntry("CustomGenres");
 	m_hideV1 = cfg.KCM_readBoolEntry("HideV1", m_hideV1);
 	m_hideV2 = cfg.KCM_readBoolEntry("HideV2", m_hideV2);
+	m_hidePicture = cfg.KCM_readBoolEntry("HidePicture", m_hidePicture);
 	m_id3v2Version = cfg.KCM_readNumEntry("ID3v2Version", static_cast<int>(ID3v2_3_0));
 	m_textEncodingV1 = cfg.readEntry("TextEncodingV1", "");
 	m_textEncoding = cfg.KCM_readNumEntry("TextEncoding", static_cast<int>(TE_ISO8859_1));
 	m_useProxy = cfg.KCM_readBoolEntry("UseProxy", m_useProxy);
 	m_proxy = cfg.readEntry("Proxy", m_proxy);
+#if QT_VERSION >= 0x040000
+	m_useProxyAuthentication = cfg.KCM_readBoolEntry("UseProxyAuthentication", m_useProxyAuthentication);
+	m_proxyUserName = cfg.readEntry("ProxyUserName", m_proxyUserName);
+	m_proxyPassword = cfg.readEntry("ProxyPassword", m_proxyPassword);
+#endif
 	m_browser = cfg.readEntry("Browser", s_defaultBrowser);
 	m_onlyCustomGenres = cfg.KCM_readBoolEntry("OnlyCustomGenres", m_onlyCustomGenres);
 
@@ -313,7 +350,9 @@ void MiscConfig::readFromConfig(
 	m_renDirSrc = config->QCM_readBoolEntry("/RenameDirectorySource", m_renDirSrc);
 	m_markTruncations = config->QCM_readBoolEntry("/MarkTruncations", m_markTruncations);
 	m_enableTotalNumberOfTracks = config->QCM_readBoolEntry("/EnableTotalNumberOfTracks", m_enableTotalNumberOfTracks);
+	m_genreNotNumeric = config->QCM_readBoolEntry("/GenreNotNumeric", m_genreNotNumeric);
 	m_preserveTime = config->QCM_readBoolEntry("/PreserveTime", m_preserveTime);
+	m_markChanges = config->QCM_readBoolEntry("/MarkChanges", m_markChanges);
 	m_commentName = config->QCM_readEntry("/CommentName", s_defaultCommentName);
 
 	m_formatText =
@@ -341,11 +380,17 @@ void MiscConfig::readFromConfig(
 	m_customGenres = config->QCM_readListEntry("/CustomGenres");
 	m_hideV1 = config->QCM_readBoolEntry("/HideV1", m_hideV1);
 	m_hideV2 = config->QCM_readBoolEntry("/HideV2", m_hideV2);
+	m_hidePicture = config->QCM_readBoolEntry("/HidePicture", m_hidePicture);
 	m_id3v2Version = config->QCM_readNumEntry("/ID3v2Version", ID3v2_3_0);
 	m_textEncodingV1 = config->QCM_readEntry("/TextEncodingV1", "");
 	m_textEncoding = config->QCM_readNumEntry("/TextEncoding", TE_ISO8859_1);
 	m_useProxy = config->QCM_readBoolEntry("/UseProxy", m_useProxy);
 	m_proxy = config->QCM_readEntry("/Proxy", m_proxy);
+#if QT_VERSION >= 0x040000
+	m_useProxyAuthentication = config->QCM_readBoolEntry("/UseProxyAuthentication", m_useProxyAuthentication);
+	m_proxyUserName = config->QCM_readEntry("/ProxyUserName", m_proxyUserName);
+	m_proxyPassword = config->QCM_readEntry("/ProxyPassword", m_proxyPassword);
+#endif
 #if defined _WIN32 || defined WIN32
 	m_browser = config->QCM_readEntry("/Browser", QString());
 	if (m_browser.isEmpty()) {
@@ -356,6 +401,8 @@ void MiscConfig::readFromConfig(
 	m_browser = config->QCM_readEntry("/Browser", s_defaultBrowser);
 #endif
 	m_onlyCustomGenres = config->QCM_readBoolEntry("/OnlyCustomGenres", m_onlyCustomGenres);
+	m_windowX = config->QCM_readNumEntry("/WindowX", -1);
+	m_windowY = config->QCM_readNumEntry("/WindowY", -1);
 	m_windowWidth = config->QCM_readNumEntry("/WindowWidth", -1);
 	m_windowHeight = config->QCM_readNumEntry("/WindowHeight", -1);
 	m_useFont = config->QCM_readBoolEntry("/UseFont", m_useFont);
@@ -388,7 +435,7 @@ void MiscConfig::readFromConfig(
 			MiscConfig::MenuCommand(
 				"AlbumArt",
 				QString('"') + prgDir +  "\\Album Cover Art Downloader\\albumart-qt.exe\" %{directory}"));
-#else
+#elif !defined __APPLE__
 		m_contextMenuCommands.push_back(
 			MiscConfig::MenuCommand("xmms", "xmms %{files}"));
 		m_contextMenuCommands.push_back(
@@ -397,7 +444,7 @@ void MiscConfig::readFromConfig(
 		m_contextMenuCommands.push_back(
 			MiscConfig::MenuCommand("Google Images", "%{browser} http://images.google.com/images?q=%u{artist}%20%u{album}"));
 		m_contextMenuCommands.push_back(
-			MiscConfig::MenuCommand("Amazon", "%{browser} http://www.amazon.com/s?field-artist=%u{artist}&field-title=%u{album}"));
+			MiscConfig::MenuCommand("Amazon", "%{browser} http://www.amazon.com/s?search-alias=aps&field-keywords=%u{artist}+%u{album}"));
 		m_contextMenuCommands.push_back(
 			MiscConfig::MenuCommand("LyricWiki", "%{browser} http://lyricwiki.org/%u{artist}:%u{title}"));
 		m_contextMenuCommands.push_back(
