@@ -160,8 +160,12 @@ public:
 	bool m_markTruncations;
 	/** true to write total number of tracks into track fields */
 	bool m_enableTotalNumberOfTracks;
+	/** true to write genres as text instead of numeric string */
+	bool m_genreNotNumeric;
 	/** true to preserve file time stamps */
 	bool m_preserveTime;
+	/** true to mark changed fields */
+	bool m_markChanges;
 	/** field name used for Vorbis comment entries */
 	QString m_commentName;
 	/** filter of file names to be opened */
@@ -199,6 +203,8 @@ public:
 	bool m_hideV1;
 	/** true to hide ID3v2.3 controls */
 	bool m_hideV2;
+	/** true to hide picture preview */
+	bool m_hidePicture;
 	/** version used for new ID3v2 tags */
 	int m_id3v2Version;
 	/** text encoding used for new ID3v1 tags */
@@ -209,11 +215,23 @@ public:
 	bool m_useProxy;
 	/** proxy used for access */
 	QString m_proxy;
+#if QT_VERSION >= 0x040000
+	/** true to use proxy authentication */
+	bool m_useProxyAuthentication;
+	/** proxy user name */
+	QString m_proxyUserName;
+	/** proxy password */
+	QString m_proxyPassword;
+#endif
 	/** web browser substituted for %b */
 	QString m_browser;
 	/** true to show only custom genres in combo boxes */
 	bool m_onlyCustomGenres;
 #ifndef CONFIG_USE_KDE
+	/** mainwindow x-position */
+	int m_windowX;
+	/** mainwindow y-position */
+	int m_windowY;
 	/** mainwindow width */
 	int m_windowWidth;
 	/** mainwindow height */
