@@ -43,6 +43,7 @@
 #include "genres.h"
 #include "dirinfo.h"
 #include "kid3.h"
+#include "attributedata.h"
 #include <cstring>
 #include <sys/stat.h>
 #ifdef WIN32
@@ -104,26 +105,26 @@ void Mp3File::readTags(bool force)
 	if (force && m_tagV1) {
 		m_tagV1->Clear();
 		m_tagV1->Link(fn, ID3TT_ID3V1);
-		markTag1Changed(false);
+		markTag1Unchanged();
 	}
 	if (!m_tagV1) {
 		m_tagV1 = new ID3_Tag;
 		if (m_tagV1) {
 			m_tagV1->Link(fn, ID3TT_ID3V1);
-			markTag1Changed(false);
+			markTag1Unchanged();
 		}
 	}
 
 	if (force && m_tagV2) {
 		m_tagV2->Clear();
 		m_tagV2->Link(fn, ID3TT_ID3V2);
-		markTag2Changed(false);
+		markTag2Unchanged();
 	}
 	if (!m_tagV2) {
 		m_tagV2 = new ID3_Tag;
 		if (m_tagV2) {
 			m_tagV2->Link(fn, ID3TT_ID3V2);
-			markTag2Changed(false);
+			markTag2Unchanged();
 		}
 	}
 
@@ -169,24 +170,24 @@ bool Mp3File::writeTags(bool force, bool* renamed, bool preserve)
 	// if there are no frames.
 	if (m_tagV1 && (force || isTag1Changed()) && (m_tagV1->NumFrames() == 0)) {
 		m_tagV1->Strip(ID3TT_ID3V1);
-		markTag1Changed(false);
+		markTag1Unchanged();
 	}
 	// Even after removing all frames, HasV2Tag() still returns true,
 	// so we strip the whole header.
 	if (m_tagV2 && (force || isTag2Changed()) && (m_tagV2->NumFrames() == 0)) {
 		m_tagV2->Strip(ID3TT_ID3V2);
-		markTag2Changed(false);
+		markTag2Unchanged();
 	}
 	// There seems to be a bug in id3lib: If I update an ID3v1 and then
 	// strip the ID3v2 the ID3v1 is removed too and vice versa, so I
 	// first make any stripping and then the updating.
 	if (m_tagV1 && (force || isTag1Changed()) && (m_tagV1->NumFrames() > 0)) {
 		m_tagV1->Update(ID3TT_ID3V1);
-		markTag1Changed(false);
+		markTag1Unchanged();
 	}
 	if (m_tagV2 && (force || isTag2Changed()) && (m_tagV2->NumFrames() > 0)) {
 		m_tagV2->Update(ID3TT_ID3V2);
-		markTag2Changed(false);
+		markTag2Unchanged();
 	}
 
 	// restore time stamp
@@ -226,7 +227,7 @@ void Mp3File::deleteFramesV1(const FrameFilter& flt)
 #else
 			delete iter;
 #endif
-			markTag1Changed();
+			markTag1Changed(Frame::FT_UnknownFrame);
 			clearTrunctionFlags();
 		} else {
 			TaggedFile::deleteFramesV1(flt);
@@ -551,17 +552,24 @@ static bool setTextField(ID3_Tag* tag, ID3_FrameID id, const QString& text,
 		ID3_Frame* frame = NULL;
 		bool removeOnly = removeEmpty && text.isEmpty();
 		if (replace || removeOnly) {
-			frame = tag->Find(id);
-			frame = tag->RemoveFrame(frame);
-			delete frame;
-			changed = true;
+			if (id == ID3FID_COMMENT && tag->HasV2Tag()) {
+				frame = tag->Find(ID3FID_COMMENT, ID3FN_DESCRIPTION, "");
+			} else {
+				frame = tag->Find(id);
+			}
+			if (frame) {
+				frame = tag->RemoveFrame(frame);
+				delete frame;
+				changed = true;
+			}
 		}
 		if (!removeOnly && (replace || tag->Find(id) == NULL)) {
 			frame = new ID3_Frame(id);
 			if (frame) {
 				ID3_Field* fld = frame->GetField(ID3FN_TEXT);
 				if (fld) {
-					ID3_TextEnc enc = Mp3File::getDefaultTextEncoding();
+					ID3_TextEnc enc = tag->HasV2Tag() ?
+						Mp3File::getDefaultTextEncoding() : ID3TE_ISO8859_1;
 					if (allowUnicode && enc == ID3TE_ISO8859_1) {
 						// check if information is lost if the string is not unicode
 						uint i, unicode_size = text.length();
@@ -629,7 +637,7 @@ static bool setYear(ID3_Tag* tag, int num)
 static bool setTrackNum(ID3_Tag* tag, int num, int numTracks = -1)
 {
 	bool changed = false;
-	if (num >= 0) {
+	if (num >= 0 && getTrackNum(tag) != num) {
 		QString str;
 		if (num != 0) {
 			str.setNum(num);
@@ -859,7 +867,7 @@ void Mp3File::setTitleV1(const QString& str)
 {
 	if (getTextField(m_tagV1, ID3FID_TITLE, s_textCodecV1) != str &&
 			setTextField(m_tagV1, ID3FID_TITLE, str, false, true, true, s_textCodecV1)) {
-		markTag1Changed();
+		markTag1Changed(Frame::FT_Title);
 		QString s = checkTruncation(str, 1 << Frame::FT_Title);
 		if (!s.isNull()) setTextField(m_tagV1, ID3FID_TITLE, s, false, true, true, s_textCodecV1);
 	}
@@ -874,7 +882,7 @@ void Mp3File::setArtistV1(const QString& str)
 {
 	if (getTextField(m_tagV1, ID3FID_LEADARTIST, s_textCodecV1) != str &&
 			setTextField(m_tagV1, ID3FID_LEADARTIST, str, false, true, true, s_textCodecV1)) {
-		markTag1Changed();
+		markTag1Changed(Frame::FT_Artist);
 		QString s = checkTruncation(str, 1 << Frame::FT_Artist);
 		if (!s.isNull()) setTextField(m_tagV1, ID3FID_LEADARTIST, s, false, true, true, s_textCodecV1);
 	}
@@ -889,7 +897,7 @@ void Mp3File::setAlbumV1(const QString& str)
 {
 	if (getTextField(m_tagV1, ID3FID_ALBUM, s_textCodecV1) != str &&
 			setTextField(m_tagV1, ID3FID_ALBUM, str, false, true, true, s_textCodecV1)) {
-		markTag1Changed();
+		markTag1Changed(Frame::FT_Album);
 		QString s = checkTruncation(str, 1 << Frame::FT_Album);
 		if (!s.isNull()) setTextField(m_tagV1, ID3FID_ALBUM, s, false, true, true, s_textCodecV1);
 	}
@@ -904,7 +912,7 @@ void Mp3File::setCommentV1(const QString& str)
 {
 	if (getTextField(m_tagV1, ID3FID_COMMENT, s_textCodecV1) != str &&
 			setTextField(m_tagV1, ID3FID_COMMENT, str, false, true, true, s_textCodecV1)) {
-		markTag1Changed();
+		markTag1Changed(Frame::FT_Comment);
 		QString s = checkTruncation(str, 1 << Frame::FT_Comment, 28);
 		if (!s.isNull()) setTextField(m_tagV1, ID3FID_COMMENT, s, false, true, true, s_textCodecV1);
 	}
@@ -918,7 +926,7 @@ void Mp3File::setCommentV1(const QString& str)
 void Mp3File::setYearV1(int num)
 {
 	if (setYear(m_tagV1, num)) {
-		markTag1Changed();
+		markTag1Changed(Frame::FT_Date);
 	}
 }
 
@@ -930,7 +938,7 @@ void Mp3File::setYearV1(int num)
 void Mp3File::setTrackNumV1(int num)
 {
 	if (setTrackNum(m_tagV1, num)) {
-		markTag1Changed();
+		markTag1Changed(Frame::FT_Track);
 		int n = checkTruncation(num, 1 << Frame::FT_Track);
 		if (n != -1) setTrackNum(m_tagV1, n);
 	}
@@ -946,7 +954,7 @@ void Mp3File::setGenreV1(const QString& str)
 	if (!str.isNull()) {
 		int num = Genres::getNumber(str);
 		if (setGenreNum(m_tagV1, num)) {
-			markTag1Changed();
+			markTag1Changed(Frame::FT_Genre);
 		}
 		// if the string cannot be converted to a number, set the truncation flag
 		checkTruncation(num == 0xff && !str.isEmpty() ? 1 : 0,
@@ -963,7 +971,7 @@ void Mp3File::setTitleV2(const QString& str)
 {
 	if (getTextField(m_tagV2, ID3FID_TITLE) != str &&
 			setTextField(m_tagV2, ID3FID_TITLE, str, true)) {
-		markTag2Changed();
+		markTag2Changed(Frame::FT_Title);
 	}
 }
 
@@ -976,7 +984,7 @@ void Mp3File::setArtistV2(const QString& str)
 {
 	if (getTextField(m_tagV2, ID3FID_LEADARTIST) != str &&
 			setTextField(m_tagV2, ID3FID_LEADARTIST, str, true)) {
-		markTag2Changed();
+		markTag2Changed(Frame::FT_Artist);
 	}
 }
 
@@ -989,7 +997,7 @@ void Mp3File::setAlbumV2(const QString& str)
 {
 	if (getTextField(m_tagV2, ID3FID_ALBUM) != str &&
 			setTextField(m_tagV2, ID3FID_ALBUM, str, true)) {
-		markTag2Changed();
+		markTag2Changed(Frame::FT_Album);
 	}
 }
 
@@ -1002,7 +1010,7 @@ void Mp3File::setCommentV2(const QString& str)
 {
 	if (getTextField(m_tagV2, ID3FID_COMMENT) != str &&
 			setTextField(m_tagV2, ID3FID_COMMENT, str, true)) {
-		markTag2Changed();
+		markTag2Changed(Frame::FT_Comment);
 	}
 }
 
@@ -1014,7 +1022,7 @@ void Mp3File::setCommentV2(const QString& str)
 void Mp3File::setYearV2(int num)
 {
 	if (setYear(m_tagV2, num)) {
-		markTag2Changed();
+		markTag2Changed(Frame::FT_Date);
 	}
 }
 
@@ -1027,7 +1035,7 @@ void Mp3File::setTrackNumV2(int num)
 {
 	int numTracks = getTotalNumberOfTracksIfEnabled();
 	if (setTrackNum(m_tagV2, num, numTracks)) {
-		markTag2Changed();
+		markTag2Changed(Frame::FT_Track);
 	}
 }
 
@@ -1039,15 +1047,19 @@ void Mp3File::setTrackNumV2(int num)
 void Mp3File::setGenreV2(const QString& str)
 {
 	if (!str.isNull()) {
-		int num = Genres::getNumber(str);
+		int num = 0xff;
+		if (!Kid3App::s_miscCfg.m_genreNotNumeric) {
+			num = Genres::getNumber(str);
+		}
 		if (num >= 0 && num != 0xff) {
-			if (setGenreNum(m_tagV2, num)) {
-				markTag2Changed();
+			if (getGenreNum(m_tagV2) != num &&
+					setGenreNum(m_tagV2, num)) {
+				markTag2Changed(Frame::FT_Genre);
 			}
 		} else {
 			if (getTextField(m_tagV2, ID3FID_CONTENTTYPE) != str &&
 					setTextField(m_tagV2, ID3FID_CONTENTTYPE, str, true)) {
-				markTag2Changed();
+				markTag2Changed(Frame::FT_Genre);
 			}
 		}
 	}
@@ -1111,11 +1123,12 @@ void Mp3File::getDetailInfo(DetailInfo& info) const
 	}
 
 	const Mp3_Headerinfo* headerInfo = 0;
-	if (m_tagV1) {
-		headerInfo = m_tagV1->GetMp3HeaderInfo();
-	} else if (m_tagV2) {
+	if (m_tagV2) {
 		headerInfo = m_tagV2->GetMp3HeaderInfo();
 	}
+	if (!headerInfo && m_tagV1) {
+		headerInfo = m_tagV1->GetMp3HeaderInfo();
+	}
 	if (headerInfo) {
 		info.valid = true;
 		switch (headerInfo->version) {
@@ -1186,11 +1199,12 @@ unsigned Mp3File::getDuration() const
 {
 	unsigned duration = 0;
 	const Mp3_Headerinfo* info = NULL;
-	if (m_tagV1) {
-		info = m_tagV1->GetMp3HeaderInfo();
-	} else if (m_tagV2) {
+	if (m_tagV2) {
 		info = m_tagV2->GetMp3HeaderInfo();
 	}
+	if (!info && m_tagV1) {
+		info = m_tagV1->GetMp3HeaderInfo();
+	}
 	if (info && info->time > 0) {
 		duration = info->time;
 	}
@@ -1539,7 +1553,9 @@ void Mp3File::setId3v2Frame(ID3_Frame* id3Frame, const Frame& frame) const
 				}
 				QString value(fld.m_value.toString());
 				if (id3Id == ID3FID_CONTENTTYPE) {
-					value = Genres::getNumberString(value, true);
+					if (!Kid3App::s_miscCfg.m_genreNotNumeric) {
+						value = Genres::getNumberString(value, true);
+					}
 				} else if (id3Id == ID3FID_TRACKNUM) {
 					addTotalNumberOfTracksIfEnabled(value);
 				}
@@ -1589,7 +1605,9 @@ bool Mp3File::setFrameV2(const Frame& frame)
 				if ((fld = id3Frame->GetField(ID3FN_TEXT)) != 0 ||
 						(fld = id3Frame->GetField(ID3FN_DESCRIPTION)) != 0) {
 					if (id3Frame->GetID() == ID3FID_CONTENTTYPE) {
-						value = Genres::getNumberString(value, true);
+						if (!Kid3App::s_miscCfg.m_genreNotNumeric) {
+							value = Genres::getNumberString(value, true);
+						}
 					} else if (id3Frame->GetID() == ID3FID_TRACKNUM) {
 						addTotalNumberOfTracksIfEnabled(value);
 					}
@@ -1615,19 +1633,77 @@ bool Mp3File::setFrameV2(const Frame& frame)
 					}
 					if (getString(fld) != value) {
 						setString(fld, value);
-						markTag2Changed();
+						markTag2Changed(frame.getType());
 					}
 					return true;
 				} else if ((fld = id3Frame->GetField(ID3FN_URL)) != 0) {
 					if (getString(fld) != value) {
 						fld->Set(value.QCM_latin1());
-						markTag2Changed();
+						markTag2Changed(frame.getType());
+					}
+					return true;
+				} else if (id3Frame->GetID() == ID3FID_PRIVATE &&
+					         (fld = id3Frame->GetField(ID3FN_DATA)) != 0) {
+					ID3_Field* ownerFld = id3Frame->GetField(ID3FN_OWNER);
+					QString owner;
+					QByteArray newData, oldData;
+					if (ownerFld && !(owner = getString(ownerFld)).isEmpty() &&
+							AttributeData(owner).toByteArray(value, newData)) {
+						QCM_duplicate(oldData,
+						              (const char *)fld->GetRawBinary(),
+						              (unsigned int)fld->Size());
+						if (newData != oldData) {
+							fld->Set(reinterpret_cast<const unsigned char*>(newData.data()),
+							         newData.size());
+							markTag2Changed(frame.getType());
+						}
+						return true;
+					}
+				} else if (id3Frame->GetID() == ID3FID_CDID &&
+					         (fld = id3Frame->GetField(ID3FN_DATA)) != 0) {
+					QByteArray newData, oldData;
+					if (AttributeData::isHexString(value, 'F', "+") &&
+							AttributeData(AttributeData::Utf16).toByteArray(value, newData)) {
+						QCM_duplicate(oldData,
+						              (const char *)fld->GetRawBinary(),
+						              (unsigned int)fld->Size());
+						if (newData != oldData) {
+							fld->Set(reinterpret_cast<const unsigned char*>(newData.data()),
+							         newData.size());
+							markTag2Changed(frame.getType());
+						}
+						return true;
+					}
+				} else if (id3Frame->GetID() == ID3FID_UNIQUEFILEID &&
+					         (fld = id3Frame->GetField(ID3FN_DATA)) != 0) {
+					QByteArray newData, oldData;
+					if (AttributeData::isHexString(value, 'Z')) {
+#if QT_VERSION >= 0x040000
+						newData = (value + '\0').toLatin1();
+#else
+						newData.duplicate((value + '\0').latin1(), value.length() + 1);
+#endif
+						QCM_duplicate(oldData,
+						              (const char *)fld->GetRawBinary(),
+						              (unsigned int)fld->Size());
+						if (newData != oldData) {
+							fld->Set(reinterpret_cast<const unsigned char*>(newData.data()),
+							         newData.size());
+							markTag2Changed(frame.getType());
+						}
+						return true;
+					}
+				} else if (id3Frame->GetID() == ID3FID_POPULARIMETER &&
+					         (fld = id3Frame->GetField(ID3FN_RATING)) != 0) {
+					if (getString(fld) != value) {
+						fld->Set(value.toInt());
+						markTag2Changed(frame.getType());
 					}
 					return true;
 				}
 			} else {
 				setId3v2Frame(id3Frame, frame);
-				markTag2Changed();
+				markTag2Changed(frame.getType());
 				return true;
 			}
 		}
@@ -1698,7 +1774,7 @@ bool Mp3File::addFrameV2(Frame& frame)
 				getFieldsFromId3Frame(id3Frame, frame.fieldList());
 				frame.setFieldListFromValue();
 			}
-			markTag2Changed();
+			markTag2Changed(frame.getType());
 			return true;
 		}
 	}
@@ -1722,7 +1798,7 @@ bool Mp3File::deleteFrameV2(const Frame& frame)
 		ID3_Frame* id3Frame = getId3v2Frame(m_tagV2, index);
 		if (id3Frame) {
 			m_tagV2->RemoveFrame(id3Frame);
-			markTag2Changed();
+			markTag2Changed(frame.getType());
 			return true;
 		}
 	}
@@ -1751,7 +1827,7 @@ void Mp3File::deleteFramesV2(const FrameFilter& flt)
 #else
 			delete iter;
 #endif
-			markTag2Changed();
+			markTag2Changed(Frame::FT_UnknownFrame);
 		} else {
 			ID3_Tag::Iterator* iter = m_tagV2->CreateIterator();
 			ID3_Frame* frame;
@@ -1769,7 +1845,7 @@ void Mp3File::deleteFramesV2(const FrameFilter& flt)
 #else
 			delete iter;
 #endif
-			markTag2Changed();
+			markTag2Changed(Frame::FT_UnknownFrame);
 		}
 	}
 }
@@ -1793,17 +1869,63 @@ void Mp3File::getAllFramesV2(FrameCollection& frames)
 			Frame frame(type, "", name, i++);
 			frame.setValue(getFieldsFromId3Frame(id3Frame, frame.fieldList()));
 			if (id3Frame->GetID() == ID3FID_USERTEXT ||
-					id3Frame->GetID() == ID3FID_WWWUSER) {
+					id3Frame->GetID() == ID3FID_WWWUSER ||
+					id3Frame->GetID() == ID3FID_COMMENT) {
+				QVariant fieldValue = frame.getFieldValue(Frame::Field::ID_Description);
+				if (fieldValue.isValid()) {
+					QString description = fieldValue.toString();
+					if (!description.isEmpty()) {
+						frame.setInternalName(QString(name) + '\n' + description);
+						frame.setType(Frame::FT_Other);
+					}
+				}
+			} else if (id3Frame->GetID() == ID3FID_PRIVATE) {
 				const Frame::FieldList& fields = frame.getFieldList();
+				QString owner;
+				QByteArray data;
 				for (Frame::FieldList::const_iterator it = fields.begin();
-						 it != fields.end();
-						 ++it) {
-					if ((*it).m_id == Frame::Field::ID_Description) {
-						QString description = (*it).m_value.toString();
-						if (!description.isEmpty()) {
-							frame.setInternalName(QString(name) + '\n' + description);
+				     it != fields.end();
+				     ++it) {
+					if ((*it).m_id == Frame::Field::ID_Owner) {
+						owner = (*it).m_value.toString();
+						if (!owner.isEmpty()) {
+							frame.setInternalName(QString(name) + '\n' + owner);
 						}
-						break;
+					} else if ((*it).m_id == Frame::Field::ID_Data) {
+						data = (*it).m_value.toByteArray();
+					}
+				}
+				if (!owner.isEmpty() && !data.isEmpty()) {
+					QString str;
+					if (AttributeData(owner).toString(data, str)) {
+						frame.setValue(str);
+					}
+				}
+			} else if (id3Frame->GetID() == ID3FID_CDID) {
+				QVariant fieldValue = frame.getFieldValue(Frame::Field::ID_Data);
+				if (fieldValue.isValid()) {
+					QString str;
+					if (AttributeData(AttributeData::Utf16).toString(fieldValue.toByteArray(), str) &&
+							AttributeData::isHexString(str, 'F', "+")) {
+						frame.setValue(str);
+					}
+				}
+			} else if (id3Frame->GetID() == ID3FID_UNIQUEFILEID) {
+				QVariant fieldValue = frame.getFieldValue(Frame::Field::ID_Data);
+				if (fieldValue.isValid()) {
+					QByteArray ba(fieldValue.toByteArray());
+					QString str(ba);
+					if (ba.size() - str.length() <= 1 &&
+							AttributeData::isHexString(str, 'Z')) {
+						frame.setValue(str);
+					}
+				}
+			} else if (id3Frame->GetID() == ID3FID_POPULARIMETER) {
+				QVariant fieldValue = frame.getFieldValue(Frame::Field::ID_Rating);
+				if (fieldValue.isValid()) {
+					QString str(fieldValue.toString());
+					if (!str.isEmpty()) {
+						frame.setValue(str);
 					}
 				}
 			}
