@@ -38,6 +38,7 @@
 #include "genres.h"
 #include "dirinfo.h"
 #include "kid3.h"
+#include "attributedata.h"
 #include <sys/stat.h>
 #ifdef WIN32
 #include <sys/utime.h>
@@ -198,8 +199,8 @@ void TagLibFile::readTags(bool force)
 #endif
 		m_tagV1 = 0;
 		m_tagV2 = 0;
-		markTag1Changed(false);
-		markTag2Changed(false);
+		markTag1Unchanged();
+		markTag2Unchanged();
 		m_fileRead = true;
 	}
 
@@ -215,56 +216,56 @@ void TagLibFile::readTags(bool force)
 		if ((mpegFile = dynamic_cast<TagLib::MPEG::File*>(file)) != 0) {
 			if (!m_tagV1) {
 				m_tagV1 = mpegFile->ID3v1Tag();
-				markTag1Changed(false);
+				markTag1Unchanged();
 			}
 			if (!m_tagV2) {
 				m_tagV2 = mpegFile->ID3v2Tag();
-				markTag2Changed(false);
+				markTag2Unchanged();
 			}
 		} else if ((flacFile = dynamic_cast<TagLib::FLAC::File*>(file)) != 0) {
 			if (!m_tagV1) {
 				m_tagV1 = flacFile->ID3v1Tag();
-				markTag1Changed(false);
+				markTag1Unchanged();
 			}
 			if (!m_tagV2) {
 				m_tagV2 = flacFile->xiphComment();
-				markTag2Changed(false);
+				markTag2Unchanged();
 			}
 #ifdef MPC_ID3V1
 		} else if ((mpcFile = dynamic_cast<TagLib::MPC::File*>(file)) != 0) {
 			if (!m_tagV1) {
 				m_tagV1 = mpcFile->ID3v1Tag();
-				markTag1Changed(false);
+				markTag1Unchanged();
 			}
 			if (!m_tagV2) {
 				m_tagV2 = mpcFile->APETag();
-				markTag2Changed(false);
+				markTag2Unchanged();
 			}
 		} else if ((wvFile = dynamic_cast<TagLib::WavPack::File*>(file)) != 0) {
 			if (!m_tagV1) {
 				m_tagV1 = wvFile->ID3v1Tag();
-				markTag1Changed(false);
+				markTag1Unchanged();
 			}
 			if (!m_tagV2) {
 				m_tagV2 = wvFile->APETag();
-				markTag2Changed(false);
+				markTag2Unchanged();
 			}
 #endif
 		} else if ((ttaFile = dynamic_cast<TagLib::TrueAudio::File*>(file)) != 0) {
 			if (!m_tagV1) {
 				m_tagV1 = ttaFile->ID3v1Tag();
-				markTag1Changed(false);
+				markTag1Unchanged();
 			}
 			if (!m_tagV2) {
 				m_tagV2 = ttaFile->ID3v2Tag();
-				markTag2Changed(false);
+				markTag2Unchanged();
 			}
 		} else {
 			m_tagV1 = 0;
-			markTag1Changed(false);
+			markTag1Unchanged();
 			if (!m_tagV2) {
 				m_tagV2 = m_fileRef.tag();
-				markTag2Changed(false);
+				markTag2Unchanged();
 			}
 		}
 	}
@@ -314,13 +315,13 @@ bool TagLibFile::writeTags(bool force, bool* renamed, bool preserve)
 			if (m_tagV1 && (force || isTag1Changed()) && m_tagV1->isEmpty()) {
 				mpegFile->strip(TagLib::MPEG::File::ID3v1);
 				fileChanged = true;
-				markTag1Changed(false);
+				markTag1Unchanged();
 				m_tagV1 = 0;
 			}
 			if (m_tagV2 && (force || isTag2Changed()) && m_tagV2->isEmpty()) {
 				mpegFile->strip(TagLib::MPEG::File::ID3v2);
 				fileChanged = true;
-				markTag2Changed(false);
+				markTag2Unchanged();
 				m_tagV2 = 0;
 			}
 			int saveMask = 0;
@@ -334,10 +335,10 @@ bool TagLibFile::writeTags(bool force, bool* renamed, bool preserve)
 				if (mpegFile->save(saveMask, false)) {
 					fileChanged = true;
 					if (saveMask & TagLib::MPEG::File::ID3v1) {
-						markTag1Changed(false);
+						markTag1Unchanged();
 					}
 					if (saveMask & TagLib::MPEG::File::ID3v2) {
-						markTag2Changed(false);
+						markTag2Unchanged();
 					}
 				}
 			}
@@ -361,20 +362,20 @@ bool TagLibFile::writeTags(bool force, bool* renamed, bool preserve)
 					if (m_tagV1 && (force || isTag1Changed()) && m_tagV1->isEmpty()) {
 						ttaFile->strip(TagLib::TrueAudio::File::ID3v1);
 						fileChanged = true;
-						markTag1Changed(false);
+						markTag1Unchanged();
 						m_tagV1 = 0;
 					}
 					if (m_tagV2 && (force || isTag2Changed()) && m_tagV2->isEmpty()) {
 						ttaFile->strip(TagLib::TrueAudio::File::ID3v2);
 						fileChanged = true;
-						markTag2Changed(false);
+						markTag2Unchanged();
 						m_tagV2 = 0;
 					}
 				}
 				if (m_fileRef.save()) {
 					fileChanged = true;
-					markTag1Changed(false);
-					markTag2Changed(false);
+					markTag1Unchanged();
+					markTag2Unchanged();
 				}
 			}
 		}
@@ -779,7 +780,7 @@ void TagLibFile::setTitleV1(const QString& str)
 				m_tagV1->setTitle(QSTRING_TO_TSTRING(s));
 			else
 				m_tagV1->setTitle(tstr);
-			markTag1Changed();
+			markTag1Changed(Frame::FT_Title);
 		}
 	}
 }
@@ -800,7 +801,7 @@ void TagLibFile::setArtistV1(const QString& str)
 				m_tagV1->setArtist(QSTRING_TO_TSTRING(s));
 			else
 				m_tagV1->setArtist(tstr);
-			markTag1Changed();
+			markTag1Changed(Frame::FT_Artist);
 		}
 	}
 }
@@ -821,7 +822,7 @@ void TagLibFile::setAlbumV1(const QString& str)
 				m_tagV1->setAlbum(QSTRING_TO_TSTRING(s));
 			else
 				m_tagV1->setAlbum(tstr);
-			markTag1Changed();
+			markTag1Changed(Frame::FT_Album);
 		}
 	}
 }
@@ -842,7 +843,7 @@ void TagLibFile::setCommentV1(const QString& str)
 				m_tagV1->setComment(QSTRING_TO_TSTRING(s));
 			else
 				m_tagV1->setComment(tstr);
-			markTag1Changed();
+			markTag1Changed(Frame::FT_Comment);
 		}
 	}
 }
@@ -857,7 +858,7 @@ void TagLibFile::setYearV1(int num)
 	if (makeTagV1Settable() && num >= 0) {
 		if (num != static_cast<int>(m_tagV1->year())) {
 			m_tagV1->setYear(num);
-			markTag1Changed();
+			markTag1Changed(Frame::FT_Date);
 		}
 	}
 }
@@ -876,7 +877,7 @@ void TagLibFile::setTrackNumV1(int num)
 				m_tagV1->setTrack(n);
 			else
 				m_tagV1->setTrack(num);
-			markTag1Changed();
+			markTag1Changed(Frame::FT_Track);
 		}
 	}
 }
@@ -893,7 +894,7 @@ void TagLibFile::setGenreV1(const QString& str)
 			TagLib::String::null : QSTRING_TO_TSTRING(str);
 		if (!(tstr == m_tagV1->genre())) {
 			m_tagV1->setGenre(tstr);
-			markTag1Changed();
+			markTag1Changed(Frame::FT_Genre);
 		}
 		// if the string cannot be converted to a number, set the truncation flag
 		checkTruncation(!str.isEmpty() && Genres::getNumber(str) == 0xff ? 1 : 0,
@@ -942,6 +943,26 @@ static TagLib::String::Type getTextEncodingConfig(bool unicode)
 	return enc;
 }
 
+/**
+ * Remove the first COMM frame with an empty description.
+ *
+ * @param id3v2Tag ID3v2 tag
+ */
+static void removeCommentFrame(TagLib::ID3v2::Tag* id3v2Tag)
+{
+	const TagLib::ID3v2::FrameList& frameList = id3v2Tag->frameList("COMM");
+	for (TagLib::ID3v2::FrameList::ConstIterator it = frameList.begin();
+			 it != frameList.end();
+			 ++it) {
+		TagLib::ID3v2::CommentsFrame* id3Frame =
+			dynamic_cast<TagLib::ID3v2::CommentsFrame*>(*it);
+		if (id3Frame && id3Frame->description().isEmpty()) {
+			id3v2Tag->removeFrame(id3Frame, true);
+			break;
+		}
+	}
+}
+
 /**
  * Write a Unicode field if the tag is ID3v2 and Latin-1 is not sufficient.
  *
@@ -958,9 +979,13 @@ bool setId3v2Unicode(TagLib::Tag* tag, const QString& qstr, const TagLib::String
 	if (tag && (id3v2Tag = dynamic_cast<TagLib::ID3v2::Tag*>(tag)) != 0) {
 		// first check if this string needs to be stored as unicode
 		TagLib::String::Type enc = getTextEncodingConfig(needsUnicode(qstr));
-		if (enc != TagLib::String::Latin1) {
-			TagLib::ByteVector id(frameId);
-			id3v2Tag->removeFrames(id);
+		TagLib::ByteVector id(frameId);
+		if (enc != TagLib::String::Latin1 || id == "COMM") {
+			if (id == "COMM") {
+				removeCommentFrame(id3v2Tag);
+			} else {
+				id3v2Tag->removeFrames(id);
+			}
 			if (!tstr.isEmpty()) {
 				TagLib::ID3v2::Frame* frame;
 				if (frameId[0] != 'C') {
@@ -1004,7 +1029,7 @@ void TagLibFile::setTitleV2(const QString& str)
 			if (!setId3v2Unicode(m_tagV2, str, tstr, "TIT2")) {
 				m_tagV2->setTitle(tstr);
 			}
-			markTag2Changed();
+			markTag2Changed(Frame::FT_Title);
 		}
 	}
 }
@@ -1023,7 +1048,7 @@ void TagLibFile::setArtistV2(const QString& str)
 			if (!setId3v2Unicode(m_tagV2, str, tstr, "TPE1")) {
 				m_tagV2->setArtist(tstr);
 			}
-			markTag2Changed();
+			markTag2Changed(Frame::FT_Artist);
 		}
 	}
 }
@@ -1042,7 +1067,7 @@ void TagLibFile::setAlbumV2(const QString& str)
 			if (!setId3v2Unicode(m_tagV2, str, tstr, "TALB")) {
 				m_tagV2->setAlbum(tstr);
 			}
-			markTag2Changed();
+			markTag2Changed(Frame::FT_Album);
 		}
 	}
 }
@@ -1061,7 +1086,7 @@ void TagLibFile::setCommentV2(const QString& str)
 			if (!setId3v2Unicode(m_tagV2, str, tstr, "COMM")) {
 				m_tagV2->setComment(tstr);
 			}
-			markTag2Changed();
+			markTag2Changed(Frame::FT_Comment);
 		}
 	}
 }
@@ -1090,7 +1115,7 @@ void TagLibFile::setYearV2(int num)
 					m_tagV2->setYear(num);
 				}
 			}
-			markTag2Changed();
+			markTag2Changed(Frame::FT_Date);
 		}
 	}
 }
@@ -1145,7 +1170,7 @@ void TagLibFile::setTrackNumV2(int num)
 					}
 				}
 			}
-			markTag2Changed();
+			markTag2Changed(Frame::FT_Track);
 		}
 	}
 }
@@ -1162,9 +1187,29 @@ void TagLibFile::setGenreV2(const QString& str)
 			TagLib::String::null : QSTRING_TO_TSTRING(str);
 		if (!(tstr == m_tagV2->genre())) {
 			if (!setId3v2Unicode(m_tagV2, str, tstr, "TCON")) {
-				m_tagV2->setGenre(tstr);
+				TagLib::ID3v2::TextIdentificationFrame* frame;
+				TagLib::ID3v2::Tag* id3v2Tag = dynamic_cast<TagLib::ID3v2::Tag*>(m_tagV2);
+				if (id3v2Tag && Kid3App::s_miscCfg.m_genreNotNumeric &&
+						(frame = new TagLib::ID3v2::TextIdentificationFrame(
+							"TCON", getDefaultTextEncoding())) != 0) {
+					frame->setText(tstr);
+					id3v2Tag->removeFrames("TCON");
+#ifdef WIN32
+					// freed in Windows DLL => must be allocated in the same DLL
+					TagLib::ID3v2::Frame* dllAllocatedFrame =
+						TagLib::ID3v2::FrameFactory::instance()->createFrame(frame->render());
+					if (dllAllocatedFrame) {
+						id3v2Tag->addFrame(dllAllocatedFrame);
+					}
+					delete frame;
+#else
+					id3v2Tag->addFrame(frame);
+#endif
+				} else {
+					m_tagV2->setGenre(tstr);
+				}
 			}
-			markTag2Changed();
+			markTag2Changed(Frame::FT_Genre);
 		}
 	}
 }
@@ -1682,7 +1727,13 @@ static QString getFieldsFromCommFrame(
 
 	field.m_id = Frame::Field::ID_Language;
 	TagLib::ByteVector bvLang = commFrame->language();
-	field.m_value = QString(QCM_QCString(bvLang.data(), bvLang.size() + 1));
+	field.m_value = QString(
+#if QT_VERSION >= 0x040000
+		QByteArray(bvLang.data(), bvLang.size())
+#else
+		QCString(bvLang.data(), bvLang.size() + 1)
+#endif
+		);
 	fields.push_back(field);
 
 	field.m_id = Frame::Field::ID_Description;
@@ -1709,7 +1760,6 @@ static QString getFieldsFromUfidFrame(
 	const TagLib::ID3v2::UniqueFileIdentifierFrame* ufidFrame,
 	Frame::FieldList& fields)
 {
-	QString text;
 	Frame::Field field;
 	field.m_id = Frame::Field::ID_Owner;
 	field.m_value = TStringToQString(ufidFrame->owner());
@@ -1722,7 +1772,14 @@ static QString getFieldsFromUfidFrame(
 	field.m_value = ba;
 	fields.push_back(field);
 
-	return text;
+	if (!ba.isEmpty()) {
+		QString text(ba);
+		if (ba.size() - text.length() <= 1 &&
+				AttributeData::isHexString(text, 'Z')) {
+			return text;
+		}
+	}
+	return QString();
 }
 
 /**
@@ -2118,6 +2175,14 @@ void setData(TagLib::ID3v2::GeneralEncapsulatedObjectFrame* f,
 	f->setObject(TagLib::ByteVector(ba.data(), ba.size()));
 }
 
+template <>
+void setData(TagLib::ID3v2::UniqueFileIdentifierFrame* f,
+						 const Frame::Field& fld)
+{
+	QByteArray ba(fld.m_value.toByteArray());
+	f->setIdentifier(TagLib::ByteVector(ba.data(), ba.size()));
+}
+
 template <class T>
 void setLanguage(T*, const Frame::Field&) {}
 
@@ -2207,6 +2272,16 @@ void setValue(TagLib::ID3v2::TextIdentificationFrame* f, const TagLib::String& t
 	setStringOrList(f, text);
 }
 
+template <>
+void setValue(TagLib::ID3v2::UniqueFileIdentifierFrame* f, const TagLib::String& text)
+{
+	if (AttributeData::isHexString(TStringToQString(text), 'Z')) {
+		TagLib::ByteVector data(text.data(TagLib::String::Latin1));
+		data.append('\0');
+		f->setIdentifier(data);
+	}
+}
+
 template <class T>
 void setText(T* f, const TagLib::String& text)
 {
@@ -2234,7 +2309,9 @@ void setTagLibFrame(const TagLibFile* self, T* tFrame, const Frame& frame)
 	if (frame.isValueChanged() || frame.getFieldList().empty()) {
 		QString text(frame.getValue());
 		if (frame.getType() == Frame::FT_Genre) {
-			text = Genres::getNumberString(text, false);
+			if (!Kid3App::s_miscCfg.m_genreNotNumeric) {
+				text = Genres::getNumberString(text, false);
+			}
 		} else if (frame.getType() == Frame::FT_Track) {
 			self->addTotalNumberOfTracksIfEnabled(text);
 		}
@@ -2250,7 +2327,9 @@ void setTagLibFrame(const TagLibFile* self, T* tFrame, const Frame& frame)
 				{
 					QString value(fld.m_value.toString());
 					if (frame.getType() == Frame::FT_Genre) {
-						value = Genres::getNumberString(value, false);
+						if (!Kid3App::s_miscCfg.m_genreNotNumeric) {
+							value = Genres::getNumberString(value, false);
+						}
 					} else if (frame.getType() == Frame::FT_Track) {
 						self->addTotalNumberOfTracksIfEnabled(value);
 					}
@@ -2543,7 +2622,7 @@ bool TagLibFile::setFrameV2(const Frame& frame)
 				// However when removing the old frame and adding a new frame,
 				// the indices of all frames get invalid.
 				setId3v2Frame(frameList[index], frame);
-				markTag2Changed();
+				markTag2Changed(frame.getType());
 				return true;
 			}
 		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
@@ -2586,7 +2665,7 @@ bool TagLibFile::setFrameV2(const Frame& frame)
 					oggTag->addField("TRACKTOTAL", TagLib::String::number(numTracks), true);
 				}
 			}
-			markTag2Changed();
+			markTag2Changed(frame.getType());
 			return true;
 		} else if ((apeTag = dynamic_cast<TagLib::APE::Tag*>(m_tagV2)) != 0) {
 			if (frame.getType() == Frame::FT_Picture) {
@@ -2594,7 +2673,7 @@ bool TagLibFile::setFrameV2(const Frame& frame)
 			}
 			apeTag->addValue(QSTRING_TO_TSTRING(getApeName(frame)),
 											 QSTRING_TO_TSTRING(frame.getValue()));
-			markTag2Changed();
+			markTag2Changed(frame.getType());
 			return true;
 		}
 	}
@@ -2715,7 +2794,7 @@ bool TagLibFile::addFrameV2(Frame& frame)
 #ifdef WIN32
 				delete id3Frame;
 #endif
-				markTag2Changed();
+				markTag2Changed(frame.getType());
 				return true;
 			}
 		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
@@ -2749,7 +2828,7 @@ bool TagLibFile::addFrameV2(Frame& frame)
 				index += (*it).second.size();
 			}
 			frame.setIndex(found ? index : -1);
-			markTag2Changed();
+			markTag2Changed(frame.getType());
 			return true;
 		} else if ((apeTag = dynamic_cast<TagLib::APE::Tag*>(m_tagV2)) != 0) {
 			if (frame.getType() == Frame::FT_Picture) {
@@ -2777,7 +2856,7 @@ bool TagLibFile::addFrameV2(Frame& frame)
 				++index;
 			}
 			frame.setIndex(found ? index : -1);
-			markTag2Changed();
+			markTag2Changed(frame.getType());
 			return true;
 		}
 	}
@@ -2805,7 +2884,7 @@ bool TagLibFile::deleteFrameV2(const Frame& frame)
 			const TagLib::ID3v2::FrameList& frameList = id3v2Tag->frameList();
 			if (index < static_cast<int>(frameList.size())) {
 				id3v2Tag->removeFrame(frameList[index]);
-				markTag2Changed();
+				markTag2Changed(frame.getType());
 				return true;
 			}
 		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
@@ -2818,12 +2897,12 @@ bool TagLibFile::deleteFrameV2(const Frame& frame)
 #else
 			oggTag->removeField(key, QSTRING_TO_TSTRING(frame.getValue()));
 #endif
-			markTag2Changed();
+			markTag2Changed(frame.getType());
 			return true;
 		} else if ((apeTag = dynamic_cast<TagLib::APE::Tag*>(m_tagV2)) != 0) {
 			TagLib::String key = QSTRING_TO_TSTRING(frame.getName(true));
 			apeTag->removeItem(key);
-			markTag2Changed();
+			markTag2Changed(frame.getType());
 			return true;
 		}
 	}
@@ -2850,7 +2929,7 @@ void TagLibFile::deleteFramesV2(const FrameFilter& flt)
 						 it != frameList.end();) {
 					id3v2Tag->removeFrame(*it++, true);
 				}
-				markTag2Changed();
+				markTag2Changed(Frame::FT_UnknownFrame);
 			} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) !=
 								 0) {
 				const TagLib::Ogg::FieldListMap& fieldListMap = oggTag->fieldListMap();
@@ -2858,14 +2937,14 @@ void TagLibFile::deleteFramesV2(const FrameFilter& flt)
 						 it != fieldListMap.end();) {
 					oggTag->removeField((*it++).first);
 				}
-				markTag2Changed();
+				markTag2Changed(Frame::FT_UnknownFrame);
 			} else if ((apeTag = dynamic_cast<TagLib::APE::Tag*>(m_tagV2)) != 0) {
 				const TagLib::APE::ItemListMap& itemListMap = apeTag->itemListMap();
 				for (TagLib::APE::ItemListMap::ConstIterator it = itemListMap.begin();
 						 it != itemListMap.end();) {
 					apeTag->removeItem((*it++).first);
 				}
-				markTag2Changed();
+				markTag2Changed(Frame::FT_UnknownFrame);
 			} else {
 				TaggedFile::deleteFramesV2(flt);
 			}
@@ -2883,7 +2962,7 @@ void TagLibFile::deleteFramesV2(const FrameFilter& flt)
 						++it;
 					}
 				}
-				markTag2Changed();
+				markTag2Changed(Frame::FT_UnknownFrame);
 			} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) !=
 								 0) {
 				const TagLib::Ogg::FieldListMap& fieldListMap = oggTag->fieldListMap();
@@ -2896,7 +2975,7 @@ void TagLibFile::deleteFramesV2(const FrameFilter& flt)
 						++it;
 					}
 				}
-				markTag2Changed();
+				markTag2Changed(Frame::FT_UnknownFrame);
 			} else if ((apeTag = dynamic_cast<TagLib::APE::Tag*>(m_tagV2)) != 0) {
 				const TagLib::APE::ItemListMap& itemListMap = apeTag->itemListMap();
 				for (TagLib::APE::ItemListMap::ConstIterator it = itemListMap.begin();
@@ -2908,7 +2987,7 @@ void TagLibFile::deleteFramesV2(const FrameFilter& flt)
 						++it;
 					}
 				}
-				markTag2Changed();
+				markTag2Changed(Frame::FT_UnknownFrame);
 			} else {
 				TaggedFile::deleteFramesV2(flt);
 			}
@@ -2939,7 +3018,8 @@ void TagLibFile::getAllFramesV2(FrameCollection& frames)
 				getTypeStringForFrameId((*it)->frameID(), type, name);
 				Frame frame(type, TStringToQString((*it)->toString()), name, i++);
 				frame.setValue(getFieldsFromId3Frame(*it, frame.fieldList(), type));
-				if ((*it)->frameID().mid(1, 3) == "XXX") {
+				if ((*it)->frameID().mid(1, 3) == "XXX" ||
+						type == Frame::FT_Comment) {
 					const Frame::FieldList& fields = frame.getFieldList();
 					for (Frame::FieldList::const_iterator it = fields.begin();
 							 it != fields.end();
@@ -2952,6 +3032,7 @@ void TagLibFile::getAllFramesV2(FrameCollection& frames)
 									description = description.mid(11);
 								}
 								frame.setInternalName(QString(name) + '\n' + description);
+								frame.setType(Frame::FT_Other);
 							}
 							break;
 						}
